//***************************************************************
// Filename: 	CLBRead.h
//
// Brief:	
// Author:	Sverre Hamre, 2009, NTNU
//***************************************************************
#ifndef __CLBRead_h__
#define __CLBRead_h__

#include <stdint.h>
#include <stdbool.h>

// Defines for Virtex IV fx12
#define VIRTEXIV                  "VirtexIV"

#define CLB_PR_ROW                24
#define CLB_PR_FRAME              16

#define FRAMES_PR_CLB             22
#define BIT_PR_CLB                80
#define BYTES_PR_CLB              BIT_PR_CLB/8

#define FRAMES_PR_DSP_INTERCON    20
#define FRAMES_PR_DSP_CONTENT     1
#define FRAMES_PR_DSP             FRAMES_PR_DSP_INTERCON + FRAMES_PR_DSP_CONTENT
#define FRAMES_PR_BRAM_INTERCON   20
#define FRAMES_PR_BRAM_CONTENT    64
#define FRAMES_PR_BRAM            FRAMES_PR_BRAM_INTERCON + FRAMES_PR_BRAM_CONTENT
#define FRAMES_PR_IOB             30
#define FRAMES_PR_CLK             3

#define FRAME_LENGTH_32BIT_WORDS  41
#define FRAME_LENGTH_BYTES        FRAME_LENGTH_32BIT_WORDS*4

#define BYTES_CENTER_WORD         4

//Configuration spesific data
//From Xilinx Datasheets remember endian!!

#define DUMMY_WORD                0xFFFFFFFF
#define SYNCHRONIZATION_WORD      0x665599AA
#define FLR_ADR                   0x01600130
#define FDRI_ADR                  0x00400030
#define FAR_ADR                   0x01200030
#define CMD_ADR                   0x01800030


// Enables debug information if defined.
// can compile with -DDEBUG_EN or uncomment
#define DEBUG_EN
#ifdef DEBUG_EN
#define DEBUG(x) x
#else
#define DEBUG(x)
#endif

// Structs
//one CLB matrix
typedef struct{
  uint8_t iCLB[BYTES_PR_CLB*FRAMES_PR_CLB]; //one CLB is 40+40 bits over 22 frames
}clb_t;

typedef struct{
  bool verbose;
  uint16_t startCLB;
  uint16_t endCLB;
  char *infilename;
  char *outfilename;
}CLBinputs_t;


// Functions

int read_frames(CLBinputs_t *inputs);

int write_frames(CLBinputs_t *inputs);

void print_CLB(CLBinputs_t *inputs);

#endif //__CLBRead_h__
