//***************************************************************
// Filename: 	main.c
//
// Brief:	Main routin for CLB read functions
//
//
// Author:	Sverre Hamre, 2009, NTNU
//
//***************************************************************

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <string.h>

#include "CLBRead.h"


// Major and minor version numbers.
#define MAJOR 0
#define MINOR 2
#define AUTHOR "Sverre Hamre"

//Prints help, update when functionality is updated.
void printhelp(void){
  printf("Usage: [comand]...\n");
  printf("\n");
  printf("-i [filename]      Input file name\n");
  printf("-o [filename]      Output file name\n");
  printf("-verbose           Explain what is beeing done\n");
  printf("-fmR               Frames mode (reads out frames)\n");
  printf("-fmW               Frames mode (writes out frames)\n");
  printf("-sc [nr]           Start CLB\n");
  printf("-ec [nr]           End CLB\n");
  printf("-version           Prints version\n");
  printf("-author            Prints author of code\n");
}

int main(int numArgs, char* argc[]){

  int i;
  int ret;
  bool noargs=false;
  bool help=false;                //Normaly do not print help.
  bool version=false;
  bool author=false;
  bool frameModeRead=false;
  bool frameModeWrite=false;
  uint32_t argCount=0;

  CLBinputs_t inputs;
  inputs.infilename = NULL;
  inputs.outfilename = NULL;
  inputs.verbose = false;


  // No arguments is given, function is run with no input.
  if(numArgs == 1){
    noargs=true;
  } else {
    for(i=1;i<numArgs;i++){
      if(*argc[i] == '-'){
	if(*(argc[i]+1) == 'h'){
	  help = true;
	  break;          //quit for print help. 
	}
	else if( strcmp((argc[i]+1), "verbose") == 0){
	  inputs.verbose = true;
	}
	else if(strcmp((argc[i]+1), "i") == 0){
	  inputs.infilename = argc[i]+3;     // store pointer to input filename
	  argCount++;
	}
	else if(strcmp((argc[i]+1), "o") == 0){
	  inputs.outfilename = argc[i]+3;    // store pointer to output filename
	  argCount++;
	}
	else if(strcmp((argc[i]+1), "sc") == 0){
	  inputs.startCLB = atoi(argc[i]+4);
	  argCount++;
	}
	else if(strcmp((argc[i]+1), "ec") == 0){
	  inputs.endCLB = atoi(argc[i]+4);
	  argCount++;
	}
	else if(strcmp((argc[i]+1), "fmR") == 0){
	  frameModeRead = true;
	}
	else if(strcmp((argc[i]+1), "fmW") == 0){
	  frameModeWrite = true;
	}
	else if(strcmp((argc[i]+1), "version") == 0){
	  version = true;              // Print out version
	}
	else if(strcmp((argc[i]+1), "author") == 0){
	  author = true;               // Print out Author name
	}
      } 
    }
  }

  if(version == true){
    printf("Version: %d.%d\n", MAJOR, MINOR);
  } 
  
  if(author == true){  
    printf("Author:  ");
    printf(AUTHOR);
    printf("\n");  
  }

  if(help == true){
    printhelp();                
  } else if(inputs.infilename && inputs.outfilename){
    if(inputs.verbose){
      printf("Input file name:       ");
      printf(inputs.infilename);
      printf("\n");
      printf("Output file name:      ");
      printf(inputs.outfilename);
      printf("\n");
      if(frameModeRead){
	printf("Frame mode, reading out frames\n");
      }else if(frameModeWrite){
	printf("Frame mode, writing out frames\n");
      }else{
	printf("bad input\n");
      }
    }
    if(frameModeRead && (argCount == 4)){ //Check all arguments present.
      ret = read_frames(&inputs);
    }else if(frameModeWrite && (argCount == 4)){ //Check all arguments present. ) 
      ret = write_frames(&inputs);
    }else{
      printf("wrong amount of arguments\n");
    }

    if(ret < 0){
      printf("Error: %d \n", ret);
    }

  } else if(noargs == true){
    printf("For help execute with -h as attribute\n");
  } else if(argCount == 2) {
    print_CLB(&inputs);
  } else if(!(version || author)) {
    printf("missing arguments\n");
    printhelp();
  }

  return 0;
}
