/* uCLinux application for accessing
 * software accessible register on the FPGA
 *
 * Author: Tormod Heimark, based on setreg.c by by Sindre Hansen
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <fcntl.h>

#define FPGADEVICE "/var/swreg" // Memory mapped FPGA

int active_NFSM(int8_t byte) { //A function that takes the output bytes from the FPGA and calculates the number of active clones
	int active = 0;
	int i;
	for ( i=1;i<5;i++){
		if( byte & (1 <<(i))){
			active++;
		}
	}
	return active;
}

int accept_o(int8_t byte){ //Function for determing if the input is accepted
	return byte & (1 <<(0));
}
		

int main(int argc, char * argv[])
{
  char readbuf[12]; //Buffers for input and output
  char writebuf[12];
  unsigned int A, B;
  int handlemem;
  int ret,i, j,n,conf_count,input_count;

  //Scanning the input file to find the number of configuration       and input vectors
  scanf("%d", &conf_count); 
  scanf("%d", &input_count);
  printf("Program for running NFSM. Program takes configuration vectors and input from textfile\n\n");

  printf("Accessing Hardware \n");
  handlemem = open(FPGADEVICE,O_RDWR); //Access through the driver
  if(handlemem == -1){
    printf("FPGA Memory could not be accessed\n");
    goto err;
  }
  else {
    printf("FPGA Memory access successful\n");
  }
  
 printf("Configuring NFSM...\n");
 for ( n=0; n < conf_count ; n++) { //Taking the vectors from the file and sending them to the software accessible register
  
  scanf("%x", &A); 
  scanf("%x", &B);
  writebuf[0] = A;
  writebuf[1] = B;
  ret = pwrite(handlemem, writebuf, 2, 0);
  
 }
  printf("NFSM configuration finished.\n");
  ret = pread(handlemem, readbuf, 4, 0);

  printf("Output from NFSM:");
  if (readbuf[2] < 16) {
 	 printf(" 0x0%X", readbuf[2]);
  } else {
	printf(" 0x0%X", readbuf[2]);
  }

  if (readbuf[3] < 16) {
 	 printf("0%X\n", readbuf[3]);
  } else {
	printf("0%X\n", readbuf[3]);
  }

  printf("Number of active NFSM clones: %d\n",active_NFSM(readbuf[3]));
  printf("Accepted input (0 no, 1 yes): %d\n\n",accept_o(readbuf[3]));

 printf("Sending input to NFSM\n");
  int last = writebuf[0];
  for ( n=0; n < input_count ; n++) { 
    scanf("%x", &A); 
    scanf("%x", &B);
    writebuf[0] = A;
    writebuf[1] = B;
    ret = pwrite(handlemem, writebuf, 2, 0);
    if (last != writebuf[0]){ //Check for avoiding unecessary print outs 
      printf("Sending input: %d \n", accept_o(writebuf[1]));
      ret = pread(handlemem, readbuf, 4, 0);
      printf("Output from NFSM:");
      if (readbuf[2] < 16) {
 	 printf(" 0x0%X", readbuf[2]);
      } else {
	printf(" 0x0%X", readbuf[2]);
      }

      if (readbuf[3] < 16) {
 	 printf("0%X\n", readbuf[3]);
      } else {
	printf("0%X\n", readbuf[3]);
      }

      printf("Number of active NFSM clones: %d\n",active_NFSM(readbuf[3]));
      printf("Accepted input (0 no, 1 yes): %d\n\n",accept_o(readbuf[3]));
      last = writebuf[0];
    }
  }

 err:
  return 0;
}
