library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


library work;
use work.check_config_package.all;

entity configuration_memory is
	generic(
		MEM_SIZE : natural  := 5;
		NUM_REG  : natural  := 20
	);
	port (
		clk	: in std_logic;
		shift_i : in std_logic_vector (MEM_SIZE-1 downto 0);
		write_en    : in std_logic;
		
		--shift_o : out std_logic_vector (MEM_SIZE-1 downto 0);
		paralell_o : out std_logic_vector(MEM_SIZE*NUM_REG-1 downto 0)
	);

end configuration_memory;

architecture arch of configuration_memory is


signal reg_inputs 	: std_logic_vector(MEM_SIZE*NUM_REG-1 downto 0);  
signal reg_outputs 	: std_logic_vector(MEM_SIZE*NUM_REG-1 downto 0); 

begin 
	OUT_PROCESS : process(reg_outputs)
	begin				
		--shift_o <= reg_outputs(MEM_SIZE-1 downto 0);
		paralell_o <= reg_outputs;
		
	end process OUT_PROCESS;

	IN_PROCESS : process(write_en,shift_i,reg_outputs) is
	begin
		reg_inputs(MEM_SIZE*NUM_REG -1 downto MEM_SIZE*(NUM_REG-1)) <= shift_i;
		for i in NUM_REG-1 downto 1 loop
			reg_inputs(MEM_SIZE*i -1 downto MEM_SIZE*(i-1)) <= reg_outputs(MEM_SIZE*(i+1) -1 downto MEM_SIZE*(i));
		end loop;  
	end process IN_PROCESS;

	REG_GEN: for i in NUM_REG downto 1 generate
		REG : entity work.paralell_shift
			generic map(C_SIZE => MEM_SIZE)
			port map(
				clk => clk,
				data_i => reg_inputs(i*MEM_SIZE-1 downto (i-1)*MEM_SIZE),
				write_en => write_en,
				data_o => reg_outputs(i*MEM_SIZE-1 downto (i-1)*MEM_SIZE)	
				);
	end generate; 

end arch;

